import merge from 'lodash.merge'
import { dibootApi } from '@/utils/request'
import moment from 'moment'
import _ from 'lodash'
export default {
  data () {
    return {
      // 请求接口基础路径
      baseApi: '/',
      // 列表数据接口
      listApi: '',
      // 删除接口
      deleteApiPrefix: '',
      // 导出接口
      exportApi: '',
      // 自定义参数（不被查询表单重置和改变的参数）
      customQueryParam: {},
      // 与查询条件绑定的参数（会被查询表单重置和改变的参数）
      queryParam: {},
      // 高级搜索 展开/关闭
      advanced: false,
      // 列表数据
      data: [],
      // 关联相关的更多数据
      more: {},
      // 获取关联数据列表的配置列表
      attachMoreList: [],
      // 是否从mixin中自动获取初始的列表数据
      getListFromMixin: true,
      // 是否使mixin在当前业务的attachMore接口中自动获取关联数据
      getMore: false,
      // 日期区间选择配置
      dateRangeQuery: {},
      // 标记加载状态
      loadingData: false,
      // 窗口高度
      windowHeight: 240,
      // 分页数据
      pagination: {
        pageSize: 10,
        current: 1,
        total: 0,
        showTotal: (total, range) => `当前显示 ${range[0]} - ${range[1]} 条/共 ${total} 条`,
        showSizeChanger: true,
        pageSizeOptions: ['10', '20', '30', '50', '100']
      }
    }
  },
  methods: {

    /**
     * 分页、排序、筛选变化时触发
     * @param pagination 分页
     * @param filters 排序
     * @param sorter 筛选
     */
    handleTableChange (pagination, filters, sorter) {
      this.queryParam.pageIndex = pagination.current
      this.queryParam.pageSize = pagination.pageSize
      this.appendSorterParam(sorter)
      this.getList()
    },
    /**
     * 构建排序
     * @param sorter
     */
    appendSorterParam (sorter) {
      if (sorter !== undefined && sorter.field !== undefined) {
        const field = sorter.field
        const order = sorter.order === 'ascend' ? 'ASC' : 'DESC'
        const orderBy = `${field}:${order}`
        this.queryParam.orderBy = orderBy
      } else {
        this.queryParam.orderBy = undefined
      }
    },
    /**
     * 切换展示更多搜索框
     */
    toggleAdvanced () {
      this.advanced = !this.advanced
    },
    /**
     * 搜索，查询第一页（默认查询按钮触发）
     */
    onSearch () {
      this.pagination.current = 1
      this.handleTableChange(this.pagination)
    },

    /**
     * post请求的获取列表（可以传递更长、更复杂参数）
     * @returns {Promise<any>}
     */
    postList () {
      this.dateRange2queryParam()
      return new Promise((resolve, reject) => {
        this.loadingData = true
        // 转化包含moment的时间类型
        this.contentTransform(this.queryParam)
        // 过滤掉不存在值的属性
        let tempQueryParam = {}
        // 合并自定义查询参数
        merge(tempQueryParam, this.customQueryParam)
        // 合并搜索参数
        merge(tempQueryParam, this.queryParam)
        // 改造查询条件（用于列表页扩展）
        tempQueryParam = this.rebuildQuery(tempQueryParam)
        // 使用post方式请求列表数据（多用于复杂参数通过json对象进行传输到后端进行筛选）
        dibootApi.post(
          this.listApi ? this.listApi : `${this.baseApi}/list`,
          tempQueryParam
        )
          .then(res => {
            this.loadingData = false
            if (res.code === 0) {
              this.data = res.data
              this.afterLoadList(this.data)
              if (res.page) {
                this.pagination.pageSize = res.page.pageSize
                this.pagination.current = res.page.pageIndex
                this.pagination.total = res.page.totalCount ? Number(res.page.totalCount) : 0
              }
              // eslint-disable-next-line no-undef
              resolve(this.data)
            } else {
              this.$notification.error({
                message: '获取列表数据失败',
                description: res.msg
              })
              reject(res.msg)
            }
          })
          .catch(err => {
            this.loadingData = false
            // eslint-disable-next-line prefer-promise-reject-errors
            this.$notification.error({
              message: '获取列表数据失败',
              description: err
            })
            reject(err)
          })
      })
    },

    /**
     * get请求获取列表
     * @returns {Promise<any>}
     */
    getList () {
      this.dateRange2queryParam()
      return new Promise((resolve, reject) => {
        this.loadingData = true
        // 转化包含moment的时间类型
        this.contentTransform(this.queryParam)
        // 过滤掉不存在值的属性
        let tempQueryParam = {}
        // 合并自定义查询参数
        merge(tempQueryParam, this.customQueryParam)
        // 合并搜索参数
        merge(tempQueryParam, this.queryParam)
        // 改造查询条件（用于列表页扩展）
        tempQueryParam = this.rebuildQuery(tempQueryParam)
        dibootApi.get(
          this.listApi ? this.listApi : `${this.baseApi}/list`,
          tempQueryParam
        ).then(res => {
          this.loadingData = false
          if (res.code === 0) {
            this.data = res.data
            this.afterLoadList(this.data)
            if (res.page) {
              this.pagination.pageSize = res.page.pageSize
              this.pagination.current = res.page.pageIndex
              this.pagination.total = res.page.totalCount ? Number(res.page.totalCount) : 0
            }
            resolve(this.data)
          } else {
            this.$notification.error({
              message: '获取列表数据失败',
              description: res.msg
            })
            reject(res.msg)
          }
        }).catch(err => {
          this.loadingData = false
          // eslint-disable-next-line prefer-promise-reject-errors
          this.$notification.error({
            message: '获取列表数据失败',
            description: err
          })
          reject(err)
        })
      })
    },
    /**
     * 重新构建查询条件 (接收已经定义的customQueryParam与queryParam的合并值)
     * @param query
     * @returns {*}
     */
    rebuildQuery (query) {
      return query
    },

    /**
     * 加载当前页面关联的对象或者字典
     * @returns {Promise<*>}
     */
    async attachMore () {
      let res = {}
      if (this.getMore === true) {
        res = await dibootApi.get(`${this.baseApi}/attachMore`)
      } else if (this.attachMoreList.length > 0) {
        res = await dibootApi.post('/common/attachMore', this.attachMoreList)
      }
      if (res.code === 0) {
        this.more = res.data
        return res.data
      }
    },
    /**
     * 重置
     */
    reset () {
      this.queryParam = {}
      this.dateRangeQuery = {}
      this.getList()
    },
    /**
     * 删除
     * @param id
     * @returns {Promise<any>}
     */
    remove (id) {
      return new Promise((resolve, reject) => {
        var _this = this
        _this.$confirm({
          title: '删除',
          content: `确定删除该数据吗？`,
          okText: '确定',
          okType: 'danger',
          cancelText: '取消',
          onOk () {
            const deleteApiPrefix = _this.deleteApiPrefix ? _this.deleteApiPrefix : ''
            dibootApi.delete(`${_this.baseApi}${deleteApiPrefix}/${id}`).then(async (res) => {
              if (res.code === 0) {
                _this.$notification.success({
                  message: '删除成功',
                  description: '已删除该数据',
                  duration: 3
                })
                await _this.getList()
                resolve(res.data)
              } else {
                _this.$notification.error({
                  message: '删除失败',
                  description: res.msg,
                  duration: 3
                })
                // eslint-disable-next-line prefer-promise-reject-errors
                reject(res.msg)
              }
            }).catch(err => {
              _this.$notification.error({
                message: '删除失败',
                description: err.msg,
                duration: 3
              })
              reject(err.msg)
            })
          }
        })
      })
    },
    /***
     * 批量删除
     * @returns {boolean}
     */
    batchRemove () {
      if (this.selectedRowKeys === undefined || this.selectedRowKeys.length === 0) {
        this.$message.error('请选择需要操作的行')
        return false
      }
      const _this = this
      this.$confirm({
        title: '您是否确认批量删除所选项？',
        onOk () {
          // 开始删除选中的行
          dibootApi.post(`${_this.baseApi}/batchDelete`, _this.selectedRowKeys).then(res => {
            if (res.code === 0) {
              _this.$message.success(res.msg)
              _this.getList()
            } else {
              _this.$message.error(res.msg)
            }
          })
        }
      })
    },
    /**
     * 导出数据至excel
     */
    exportData () {
      // 转化包含moment的时间类型
      this.contentTransform(this.queryParam)
      let tempQueryParam = {}
      // 合并自定义查询参数
      merge(tempQueryParam, this.customQueryParam)
      // 合并搜索参数
      merge(tempQueryParam, this.queryParam)
      // 改造查询条件（用于列表页扩展）
      tempQueryParam = this.rebuildQuery(tempQueryParam)
      const exportApi = this.exportApi ? this.exportApi : '/excel/export'
      dibootApi.download(`${this.baseApi}${exportApi}`, tempQueryParam).then(res => {
        if (res.filename) {
          this.downloadFile(res)
        } else {
          var decoder = new TextDecoder('utf-8')
          var result = JSON.parse(decoder.decode(new Uint8Array(res)))
          this.$message.error(result.msg)
        }
      })
    },
    /**
     * 下载文件
     * @param res
     */
    downloadFile (res) {
      const blob = new Blob([res.data])
      if ('download' in document.createElement('a')) { // 非IE下载
        const elink = document.createElement('a')
        elink.download = res.filename
        elink.style.display = 'none'
        elink.href = URL.createObjectURL(blob)
        document.body.appendChild(elink)
        elink.click()
        URL.revokeObjectURL(elink.href) // 释放URL 对象
        document.body.removeChild(elink)
      } else { // IE10+下载
        navigator.msSaveBlob(blob, res.filename)
      }
    },
    /**
     * 加载数据之后操作
     * @param list
     */
    afterLoadList (list) {

    },

    /**
     * 解决带有下拉框组件在滚动时下拉框不随之滚动的问题
     * @param trigger
     * @returns {HTMLElement}
     */
    getPopupContainer (trigger) {
      return trigger.parentElement
    },
    /**
     * 处理查询参数中的moment数据 默认转化为YYYY-MM-DD
     * 如果需要单独处理属性，那么请传入transform对象，指定属性的转化类型
     * 示例：{createTime: 'YYYY-MM-DD'}
     * @param content 待转化内容
     * @param transform 需要转化的格式
     */
    contentTransform (content, transform = {}) {
      if (content) {
        for (const key in content) {
          const value = content[key]
          const format = transform[key] || 'YYYY-MM-DD'
          if (value instanceof Array) {
            // 如果类型是moment，那么进行转化
            if (value && value[0] instanceof moment) {
              const transformTime = []
              for (let i = 0; i < value.length; i++) {
                transformTime[i] = value[i].format(format)
              }
              content[key] = transformTime
            }
          } else if (value instanceof moment) {
            content[key] = value.format(format)
          }
        }
      }
      return content
    },
    /**
     * 构建区间查询参数
     */
    dateRange2queryParam () {
      _.forEach(this.dateRangeQuery, (v, k) => {
        if (k && v && v.length === 2) {
          this.queryParam[`${k}Begin`] = v[0].format('YYYY-MM-DD')
          this.queryParam[`${k}End`] = v[1].format('YYYY-MM-DD')
        }
      })
    },
    windowResize () {
      // 设置workPanel的高度
      this.windowHeight = window.innerHeight
    }
  },
  async mounted () {
    if (this.getListFromMixin === true) {
      await this.getList()
    }
    await this.attachMore()
  },
  computed: {
    tableScrollData: function () {
      return { x: 'calc(700px + 50%)', y: `${this.windowHeight > 560 ? this.windowHeight - 420 : 1200}px` }
    }
  },
  created () {
    // 设置workPanel的高度
    this.windowResize(false)
    window.addEventListener('resize', this.windowResize)
  },
  destroyed () {
    window.removeEventListener('resize', this.windowResize)
  }
}
